@@ -2,11 +2,14 @@ from matplotlib.matlab import *
 
 t = arange(0.0, 2.0, 0.01)
 s = sin(2*pi*t)
-plot(t, s)
+l = plot(t, s)
+
 xlabel('time (s)')
 ylabel('voltage (mV)')
-title('About as simple as it gets, folks')
+t = title('About as simple as it gets, folks')
+
+
 #grid(True)
 #set(gca(), 'xticks', (0,.2,.7))
-#savefig('test2', dpi=200)
+#savefig('test2')
 show()
