@@ -11,7 +11,7 @@ from artist import Artist, DPI
 from axes import Axes, Subplot
 from patches import Rectangle
 from cbook import is_string_like, enumerate, True, False
-from text import _process_text_args
+from text import Text, _process_text_args
 from transforms import Bound1D, Bound2D, Transform, BinOp
 
 colors = {
