@@ -9,6 +9,8 @@ Plotting commands
   axis     - Set or return the current axis limits
   bar      - make a bar chart
   close    - close a figure window
+  cohere   - make a plot of coherence
+  csd      - make a plot of cross spectral density
   errorbar - make an errorbar graph
   figure   - create or change active figure
   gca      - return the current axes
@@ -16,7 +18,9 @@ Plotting commands
   get      - get a handle graphics property
   hist     - make a histogram
   loglog   - a log log plot
+  pcolor   - make a pseudocolor plot
   plot     - make a line plot
+  psd      - make a plot of power spectral density
   savefig  - save the current figure
   scatter  - make a scatter plot
   set      - set a handle graphics property
@@ -113,15 +117,16 @@ from MLab import *
 from mlab import *
 from FFT import fft
 
-from cbook import is_string_like, flatten
-from axes import Subplot, Axes
+import _matlab_helpers
 import mlab  #so I can override hist, psd, etc...
 
-from backends import Figure, Gcf, AxisText,  \
-     error_msg, draw_if_interactive, show
-from patches import Rectangle
+from axes import Subplot, Axes
+from backends import Figure, new_figure_manager, error_msg, \
+     draw_if_interactive, show
+from cbook import flatten, is_string_like
 from colormap import ColormapJet
-from cbook import flatten
+from lines import Line2D
+from patches import Rectangle
 
 def colors():
     """
@@ -168,15 +173,22 @@ def plotting():
     axis     - Set or return the current axis limits
     bar      - make a bar chart
     close    - close a figure window
+    cohere   - make a plot of coherence
+    csd      - make a plot of cross spectral density
     errorbar - make an errorbar graph
     figure   - create or change active figure
     gca      - return the current axes
     gcf      - return the current figure
     get      - get a handle graphics property
     hist     - make a histogram
+    loglog   - a log log plot
+    pcolor   - make a pseudocolor plot
     plot     - make a line plot
+    psd      - make a plot of power spectral density
     savefig  - save the current figure
     scatter  - make a scatter plot
+    semilogx - log x axis
+    semilogy - log y axis
     set      - set a handle graphics property
     show     - show the figures
     subplot  - make a subplot (numrows, numcols, axesnum)
@@ -189,10 +201,18 @@ def plotting():
 
 
 
-def get_plot_commands():
-    return ['axis', 'axes', 'subplot', 'plot', 'set', 'get', 'title',
-     'xlabel', 'ylabel', 'text', 'scatter', 'errorbar', 'bar', 'hist',
-     'figure', 'gca', 'gcf', 'close' ]
+
+def get_current_fig_manager():
+    figManager = _matlab_helpers.Gcf.get_active()
+    if figManager is None:
+        gcf()  # creates an active figure as a side effect
+        figManager = _matlab_helpers.Gcf.get_active()
+    return figManager
+
+def get_plot_commands(): return ['axis', 'axes', 'subplot', 'plot',
+    'set', 'get', 'title', 'xlabel', 'ylabel', 'text', 'scatter',
+    'errorbar', 'bar', 'hist', 'figure', 'gca', 'gcf', 'close',
+    'pcolor', 'loglog', 'semilogx', 'semilogy', 'psd', 'csd', 'cohere']
 
 
 def raise_msg_to_str(msg):
@@ -202,79 +222,6 @@ def raise_msg_to_str(msg):
     return msg
 
 #----- Now we get started with the matlab commands  ----#
-        
-def gcf():
-    "Return a handle to the current figure"
-    return Gcf().get_current_figwin().figure
-
-def figure(num=1):
-    """
-    Create a new figure and return a handle to it
-
-    If figure(num) already exists, make it active and return the
-    handle to it.
-
-      figure(1)
-
-    """
-    if num==0:
-        error_msg('Figure number can not be 0.\n' + \
-                  'Hey, give me a break, this is matlab compatability')
-
-    return Gcf(num).get_current_figwin().figure
-    
-def close(*args):
-    """
-y    Close a figure window
-
-    close() by itself closes the current figure
-
-    close(num) closes figure number num
-
-    close(h) where h is a figure handle(instance) closes that figure
-
-    close('all') closes all the figure windows
-    """
-
-    if len(args)==0:
-        figwin = Gcf().get_current_figwin()
-        Gcf.destroy(figwin.num)
-    elif len(args)==1:
-        arg = args[0]
-        if arg=='all':
-            for figwin in Gcf.get_all_figwins():
-                Gcf.destroy(figwin.num)
-        elif isinstance(arg, int):
-            Gcf.destroy(arg)
-        elif isinstance(arg, Figure):
-            for figwin in Gcf.get_all_figwins():
-                if figwin.figure==arg:
-                    Gcf.destroy(figwin.num)
-        else:
-            msg = 'Unrecognized argument type to close'
-            popup_err(msg)
-    else:
-        msg = 'close takes 0 or 1 arguments'
-        popup_err(msg)
-
-def gca():
-    """
-    Return the current axis instance.  This can be used to control
-    axis properties either using set or the Axes methods.
-
-    Example:
-
-      plot(t,s)
-      set(gca(), 'xlim', [0,10])  # set the x axis limits
-
-    or
-
-      plot(t,s)
-      a = gca()
-      a.set_xlim([0,10])          # does the same
-    """
-
-    return Gcf().get_current_figwin().get_current_axis()
 
 def axis(*v):
     """
@@ -310,44 +257,41 @@ def axes(*args, **kwargs):
     """
     Add an axis at positon rect specified by
 
-      axes() by itself creates a default full window axis
+      axes() by itself creates a default full subplot(111) window axis
 
       axes(rect, axisbg='w') where rect=[left, bottom, width, height] in
         normalized (0,1) units background is the background color for
         the axis, default white
 
-      axes(h, axisbg='w') where h is an axes instance makes h the
+      axes(h) where h is an axes instance makes h the
         current axis An Axes instance is returned
 
     axisbg is a color format string which sets the background color of
-    the axes (default white).
+    the axes
 
     If axisbg is a length 1 string, assume it's a color format string
     (see plot for legal color strings).  If it is a length 7 string,
     assume it's a hex color string, as used in html, eg, '#eeefff'.
     If it is a len(3) tuple, assume it's an rgb value where r,g,b in
-    (0,255).
+    [0,1].
     
     """
 
     nargs = len(args)
-    if args==0: return subplot(111, **kwargs)
+    if len(args)==0: return subplot(111, **kwargs)
     if nargs>1:
         error_msg('Only one non keyword arg to axes allowed')
-
+        return
+    
     arg = args[0]
 
     if isinstance(arg, Axes):
-        Gcf().get_current_figwin().set_current_axes(arg)
+        get_current_fig_manager().set_current_axes(arg)
         return arg
     else:
-        if len(args)==2:
-            rect, bg = args
-        else:
-            rect = arg
-            bg = kwargs.get('axisbg', 'w')
-        #print 'rect', arg
-        ret =  Gcf().get_current_figwin().add_axes(rect, bg)
+        rect = arg
+        bg = kwargs.get('axisbg', 'w')
+        ret = get_current_fig_manager().add_axes(rect, bg)
         return ret
 
 
@@ -355,9 +299,21 @@ def bar(*args, **kwargs):
     """
     bar(self, x, y, width=0.8)
 
-    Make a bar plot with rectangles at x, x+width, 0, y
-    x and y are Numeric arrays
+      Make a bar plot with rectangles at x, x+width, 0, y x and y are
+      Numeric arrays
 
+    bar(x, y, width=0.8, color='b', yerr=None, xerr=None, capsize=3)
+
+      xerr and yerr, if not None, will be used to generate errorbars
+      on the bar chart for the x and y axes, respectively.  See
+      errorbar for more info on the xerr and yerr optional arguments.
+
+      color can be a matplotlib color string or tuple, or it can be
+      a len(x) list of matplotlib color strings or tuples
+
+      capsize gives the length of the errorbar; see errorbar for more
+      info.
+      
     Return value is a list of Rectangle patch instances
     """
 
@@ -369,60 +325,356 @@ def bar(*args, **kwargs):
     draw_if_interactive()
     return patches
 
-def errorbar(x, y, e, u=None, fmt='b-'):
+def close(*args):
+    """
+    Close a figure window
+
+    close() by itself closes the current figure
+
+    close(num) closes figure number num
+
+    close(h) where h is a figure handle(instance) closes that figure
+
+    close('all') closes all the figure windows
     """
 
-    Plot x versus y with error bars in e.  if u is not None, then u
-    gives the upper error bars and e gives the lower error bars.
-    Otherwise e the error bars are symmetrix about y and given in the
-    array e.
+    if len(args)==0:
+        figManager = _matlab_helpers.Gcf.get_active()
+        if figManager is None: return
+        else: _matlab_helpers.Gcf.destroy(figManager.num)
+    elif len(args)==1:
+        arg = args[0]
+        if arg=='all':
+            for manager in _matlab_helpers.Gcf.get_all_fig_managers():
+                _matlab_helpers.Gcf.destroy(manager.num)
+        elif isinstance(arg, int):
+            _matlab_helpers.Gcf.destroy(arg)
+        elif isinstance(arg, Figure):
+            for manager in _matlab_helpers.Gcf.get_all_fig_managers():
+                if manager.figure==arg:
+                    _matlab_helpers.Gcf.destroy(manager.num)
+        else:
+            msg = 'Unrecognized argument type to close'
+            popup_err(msg)
+    else:
+        msg = 'close takes 0 or 1 arguments'
+        popup_err(msg)
+
+def cohere(x, y, NFFT=256, Fs=2, detrend=mlab.detrend_none,
+           window=mlab.window_hanning, noverlap=0):
+    """
+    Compute the coherence between x and y.  Coherence is the
+    normalized cross spectral density
+
+    Cxy = |Pxy|^2/(Pxx*Pyy)
+
+    The return value is (Cxy, f), where f are the frequencies of the
+    coherence vector.  See the docs for psd and csd for information
+    about the function arguments NFFT, detrend, windowm noverlap, as
+    well as the methods used to compute Pxy, Pxx and Pyy.
+
+    Returns the tuple Cxy, freqs
+
+    Refs:
+      Bendat & Piersol -- Random Data: Analysis and Measurement
+        Procedures, John Wiley & Sons (1986)
+
+    """
+
+    try: ret =  gca().cohere(x, y, NFFT, Fs, detrend, window, noverlap)
+    except ValueError, msg:
+        msg = raise_msg_to_str(msg)
+        error_msg(msg)
+    else:
+        draw_if_interactive()
+        return ret
+
+
+def csd(x, y, NFFT=256, Fs=2, detrend=mlab.detrend_none,
+        window=mlab.window_hanning, noverlap=0):
+    """
+    The cross spectral density Pxy by Welches average periodogram
+    method.  The vectors x and y are divided into NFFT length
+    segments.  Each segment is detrended by function detrend and
+    windowed by function window.  noverlap gives the length of the
+    overlap between segments.  The product of the direct FFTs of x and
+    y are averaged over each segment to compute Pxy, with a scaling to
+    correct for power loss due to windowing.  Fs is the sampling
+    frequency.
+
+    NFFT must be a power of 2
+
+    detrend and window are functions, unlike in matlab where they are
+    vectors.  For detrending you can use detrend_none, detrend_mean,
+    detrend_linear or a custom function.  For windowing, you can use
+    window_none, window_hanning, or a custom function
+
+    Returns the tuple Pxy, freqs.  Pxy is the cross spectrum (complex
+    valued), and 10*log10(|Pxy|) is plotted
+
+    Refs:
+      Bendat & Piersol -- Random Data: Analysis and Measurement
+        Procedures, John Wiley & Sons (1986)
+
+    """
+
+    try: ret =  gca().csd(x, y, NFFT, Fs, detrend, window, noverlap)
+    except ValueError, msg:
+        msg = raise_msg_to_str(msg)
+        error_msg(msg)
+    else:
+        draw_if_interactive()
+        return ret
+
+def errorbar(x, y, yerr=None, xerr=None, fmt='b-', capsize=3):
+    """
+    Plot x versus y with error deltas in yerr and xerr.
+    Vertical errorbars are plotted if yerr is not None
+    Horizontal errorbars are plotted if xerr is not None
+
+    xerr and yerr may be any of:
+        a rank-0, Nx1 Numpy array  - symmetric errorbars +/- value
+        an N-element list or tuple - symmetric errorbars +/- value
+        a rank-1, Nx2 Numpy array  - asymmetric errorbars -column1/+column2
     
     fmt is the plot format symbol for y
 
     Return value is a length 2 tuple.  The first element is a list of
     y symbol lines.  The second element is a list of error bar lines.
-    
-    """
-    
-    l0 = plot(x,y,fmt)
 
-    e = asarray(e)
-    if u is None: u = e
-    upper = y+u
-    lower = y-e
-    width = (max(x)-min(x))*0.001
-    a = gca()
-    try: 
-        l1 = a.vlines(x, y, lower)
-        l2 = a.vlines(x, y, upper)
-        l3 = a.hlines(upper, x-width, x+width)
-        l4 = a.hlines(lower, x-width, x+width)
+    capsize is the size of the error bar caps in points
+    """
+    try: ret =  gca().errorbar(x, y, yerr, xerr, fmt, capsize)
     except ValueError, msg:
         msg = raise_msg_to_str(msg)
         error_msg(msg)
-        raise RuntimeError, msg
+    else:
+        draw_if_interactive()
+        return ret
+
+def figure(num=1, figsize=(8,6), dpi=72):
+    """
+    Create a new figure and return a handle to it
+
+    If figure(num) already exists, make it active and return the
+    handle to it.
+
+      figure(1)
+
+    """
+    if num==0:
+        error_msg('Figure number can not be 0.\n' + \
+                  'Hey, give me a break, this is matlab compatability')
+
+    figManager = _matlab_helpers.Gcf.get_fig_manager(num)
+    if figManager is None:
+        figManager = new_figure_manager(num, figsize, dpi)
+        _matlab_helpers.Gcf.set_active(figManager)
     
-    l1.extend(l2)
-    l3.extend(l4)
-    l1.extend(l3)
-    draw_if_interactive()
-    return (l0, l1)
+    return figManager.figure
 
 
+def gca():
+    """
+    Return the current axis instance.  This can be used to control
+    axis properties either using set or the Axes methods.
+
+    Example:
+
+      plot(t,s)
+      set(gca(), 'xlim', [0,10])  # set the x axis limits
+
+    or
+
+      plot(t,s)
+      a = gca()
+      a.set_xlim([0,10])          # does the same
+    """
+
+    return get_current_fig_manager().get_current_axis()
+        
+def gcf():
+    "Return a handle to the current figure"
+    
+    figManager = _matlab_helpers.Gcf.get_active()
+    if figManager is not None:
+        return figManager.figure
+    else:
+        return figure()
 
 def get(o, s):
     """
     Return the value of handle property s
 
-    h is an instance of a class, eg a Line2D or an Axes or AxisText.
+    h is an instance of a class, eg a Line2D or an Axes or Text.
     if s is 'somename', this function returns
 
       o.get_somename()
     
     """
-    func = 'o.get_%s(val)' % s
+    func = 'o.get_%s()' % s
     return eval(func, {}, {'o': o})
+    
 
+def grid(b):
+    """
+    Set the figure grid to be on or off (b is a boolean)
+    """
+    gca().grid(b)
+
+def hist(x, bins=10, noplot=0, normed=0):
+    """
+    Compute the histogram of x.  bins is either an integer number of
+    bins or a sequence giving the bins.  x are the data to be binned.
+
+    if noplot is True, just compute the histogram and return the
+    number of observations and the bins as an (n, bins) tuple.
+
+    If noplot is False, compute the histogram and plot it, returning
+    n, bins, patches
+
+    If normed is true, the first element of the return tuple will be the
+    counts normalized to form a probability distribtion, ie,
+    n/(len(x)*dbin)
+    
+    """
+    if noplot: return mlab.hist(x, bins, normed)
+    else:
+        try:
+            ret = gca().hist(x, bins, normed)
+        except ValueError, msg:
+            msg = raise_msg_to_str(msg)
+            error_msg(msg)
+            raise RuntimeError, msg
+    draw_if_interactive()
+    return ret
+
+    
+
+def hlines(*args, **kwargs):    
+    """
+    lines = hlines(self, y, xmin, xmax, fmt='k-')
+
+    plot horizontal lines at each y from xmin to xmax.  xmin or
+    xmax can be scalars or len(x) numpy arrays.  If they are
+    scalars, then the respective values are constant, else the
+    widths of the lines are determined by xmin and xmax
+
+    Returns a list of line instances that were added
+
+    """
+    try: lines =  gca().hlines(*args, **kwargs)
+    except ValueError, msg:
+        msg = raise_msg_to_str(msg)
+        error_msg(msg)
+        raise RuntimeError, msg
+    draw_if_interactive()
+    return lines
+
+
+def legend(*args, **kwargs):
+    """
+    Place a legend on the current axes at location loc.  Labels are a
+    sequence of strings and loc can be a string or an integer
+    specifying the legend location
+
+    USAGE: 
+
+      Make a legend with existing lines
+      legend( LABELS )
+      >>> legend( ('label1', 'label2', 'label3') ) 
+
+      Make a legend for Line2D instances lines1, line2, line3
+      legend( LINES, LABELS )
+      >>> legend( (line1, line2, line3), ('label1', 'label2', 'label3') )
+
+      Make a legend at LOC
+      legend( LABELS, LOC )  or
+      legend( LINES, LABELS, LOC )
+      >>> legend( ('label1', 'label2', 'label3'), loc='upper left')
+      >>> legend( (line1, line2, line3),
+                  ('label1', 'label2', 'label3'),
+                  loc=2)
+
+    The LOC location codes are
+
+    The location codes are
+
+      'best' : 0,          (currently not supported, defaults to upper right)
+      'upper right'  : 1,  (default)
+      'upper left'   : 2,
+      'lower left'   : 3,
+      'lower right'  : 4,
+      'right'        : 5,
+      'center left'  : 6,
+      'center right' : 7,
+      'lower center' : 8,
+      'upper center' : 9,
+      'center'       : 10,
+
+    Return value is a sequence of text, line instances that make
+    up the legend
+    """
+
+    gca().legend(*args, **kwargs)
+
+
+def loglog(*args, **kwargs):
+    """
+    Make a loglog plot with log scaling on the a and y axis.  The args
+    to semilog x are the same as the args to plot.  See help plot for
+    more info
+    """
+    try: ret =  gca().loglog(*args, **kwargs)
+    except ValueError, msg:
+        msg = raise_msg_to_str(msg)
+        error_msg(msg)
+    else:
+        draw_if_interactive()
+        return ret
+
+
+def pcolor(*args, **kwargs):
+    """
+    pcolor(C) - make a pseudocolor plot of matrix C
+
+    pcolor(X, Y, C) - a pseudo color plot of C on the matrices X and Y  
+
+    Shading:
+
+      The optional keyword arg shading ('flat' or 'faceted') will
+      determine whether the black grid is drawn around each pcolor
+      square.  Defaul 'faceteted'
+         e.g.,   
+         pcolor(C, shading='flat')  
+         pcolor(X, Y, C, shading='faceted')
+
+    returns a list of patch objects.
+
+    Note, the behavior of meshgrid in matlab is a bit
+    counterintuitive for x and y arrays.  For example,
+
+      x = arange(7)
+      y = arange(5)
+      X, Y = meshgrid(x,y)
+
+      Z = rand( len(x), len(y))
+      pcolor(X, Y, Z)
+
+    will fail in matlab and matplotlib.  You will probably be
+    happy with
+
+      pcolor(X, Y, transpose(Z))
+
+    Likewise, for nonsquare Z,
+
+      pcolor(transpose(Z))
+
+    will make the x and y axes in the plot agree with the numrows
+    and numcols of Z
+
+    """
+    return gca().pcolor(*args, **kwargs)
 
 def plot(*args, **kwargs):
     """
@@ -481,58 +733,52 @@ def plot(*args, **kwargs):
         draw_if_interactive()
         return lines
 
-def hist(x, bins=10, noplot=0, normed=0):
+
+def psd(x, NFFT=256, Fs=2, detrend=mlab.detrend_none,
+        window=mlab.window_hanning, noverlap=0):
     """
-    Compute the histogram of x.  bins is either an integer number of
-    bins or a sequence giving the bins.  x are the data to be binned.
+    The power spectral density by Welches average periodogram method.
+    The vector x is divided into NFFT length segments.  Each segment
+    is detrended by function detrend and windowed by function window.
+    noperlap gives the length of the overlap between segments.  The
+    absolute(fft(segment))**2 of each segment are averaged to compute Pxx,
+    with a scaling to correct for power loss due to windowing.  Fs is
+    the sampling frequency.
 
-    if noplot is True, just compute the histogram and return the
-    number of observations and the bins as an (n, bins) tuple.
+    -- NFFT must be a power of 2
 
-    If noplot is False, compute the histogram and plot it, returning
-    n, bins, patches
-
-    If normed is true, the first element of the return tuple will be the
-    counts normalized to form a probability distribtion, ie,
-    n/(len(x)*dbin)
-    
-    """
-    n,bins = mlab.hist(x, bins, normed)
-    width = 0.9*(bins[1]-bins[0])
-    if noplot: return n, bins
-    else:
-        try:
-            patches = gca().bar(bins, n, width=width)
-        except ValueError, msg:
-            msg = raise_msg_to_str(msg)
-            error_msg(msg)
-            raise RuntimeError, msg
-    draw_if_interactive()
-    return n, bins, patches
+    -- detrend and window are functions, unlike in matlab where they
+       are vectors.  For detrending you can use detrend_none,
+       detrend_mean, detrend_linear or a custom function.  For
+       windowing, you can use window_none, window_hanning, or a custom
+       function
 
+    -- if length x < NFFT, it will be zero padded to NFFT
     
 
-def hlines(*args, **kwargs):    
-    """
-    lines = hlines(self, y, xmin, xmax, fmt='k-')
+    Returns the tuple Pxx, freqs
 
-    plot horizontal lines at each y from xmin to xmax.  xmin or
-    xmax can be scalars or len(x) numpy arrays.  If they are
-    scalars, then the respective values are constant, else the
-    widths of the lines are determined by xmin and xmax
-
-    Returns a list of line instances that were added
+    For plotting, the power is plotted as 10*log10(pxx)) for decibels,
+    though pxx itself is returned
+    
+    Refs:
+      Bendat & Piersol -- Random Data: Analysis and Measurement
+        Procedures, John Wiley & Sons (1986)
 
     """
-    try: lines =  gca().hlines(*args, **kwargs)
+    try: ret =  gca().psd(x, NFFT, Fs, detrend, window, noverlap)
     except ValueError, msg:
         msg = raise_msg_to_str(msg)
         error_msg(msg)
-        raise RuntimeError, msg
-    draw_if_interactive()
-    return lines
+    else:
+        draw_if_interactive()
+        return ret
+    
+        
 
-def savefig(fname, dpi=300):
+
+
+def savefig(fname, dpi=150):
     """
     Save the current figure to filename fname.  dpi is the resolution
     in dots per inch.
@@ -567,34 +813,80 @@ def scatter(*args, **kwargs):
     draw_if_interactive()
     return patches
 
-def set(h, s, val):
+
+
+def semilogx(*args, **kwargs):
+    """
+    Make a semilog plot with log scaling on the x axis.  The args to
+    semilog x are the same as the args to plot.  See help plot for
+    more info    
+    """
+    try: ret =  gca().semilogx(*args, **kwargs)
+    except ValueError, msg:
+        msg = raise_msg_to_str(msg)
+        error_msg(msg)
+    else:
+        draw_if_interactive()
+        return ret
+
+
+def semilogy(*args, **kwargs):
+    """
+    Make a semilog plot with log scaling on the y axis.  The args to
+    semilog x are the same as the args to plot.  See help plot for
+    more info    
+    """
+    try: ret =  gca().semilogy(*args, **kwargs)
+    except ValueError, msg:
+        msg = raise_msg_to_str(msg)
+        error_msg(msg)
+    else:
+        draw_if_interactive()
+        return ret
+        
+    
+def set(h, *args, **kwargs):
     """
     Set handle h property in string s to value val
 
     h can be a handle or vector of handles.
 
     h is an instance (or vector of instances) of a class, eg a Line2D
-    or an Axes or AxisText.  if s is 'somename', this function calls
+    or an Axes or Text.
 
-      o.set_somename(val)
+    args is a list of string, value pairs.  if the string
+    is 'somename', set function calls
 
-    for every instance in o in h
+      o.set_somename(value)
+
+    for every instance in h.
+
+    
     """
-    s = s.lower()
+
     if not iterable(h): h = [h]
     else: h = flatten(h)
+
+    if len(args)%2:
+        error_msg('The set args must be string, value pairs')
+
+    funcvals = []
+    for i in range(0, len(args)-1, 2):
+        funcvals.append((args[i], args[i+1]))
+    funcvals.extend(kwargs.items())
+
     for o in h:
-        funcName = "set_%s"%s
-        func = getattr(o,funcName)        
-        try: func(val)
-        except ValueError, msg:
-            msg = raise_msg_to_str(msg)
-            error_msg(msg)
-            raise RuntimeError, msg
+        for s, val in funcvals:
+            s = s.lower()
+            funcName = "set_%s"%s
+            func = getattr(o,funcName)        
+            try: func(val)
+            except ValueError, msg:
+                msg = raise_msg_to_str(msg)
+                error_msg(msg)
+                raise RuntimeError, msg
     draw_if_interactive()
 
-
-
 def subplot(*args, **kwargs):
     """
     Create a subplot command, creating axes with
@@ -617,7 +909,7 @@ def subplot(*args, **kwargs):
     subplot(211, axisbg='y')
     """
     try:
-        Gcf().get_current_figwin().add_subplot(*args, **kwargs)
+        get_current_fig_manager().add_subplot(*args, **kwargs)
         a =  gca()
     except ValueError, msg:
         msg = raise_msg_to_str(msg)
@@ -626,19 +918,18 @@ def subplot(*args, **kwargs):
     draw_if_interactive()
     return a
 
-def text(x, y, label, *args, **kwargs):
+def text(x, y, label, fontdict=None, **kwargs):
     """
     Add text to axis at location x,y
 
-    args, if present, must be a single argument which is a dictionary
-    to override the default text properties.  If no dictionary is
-    provided, this will be used
+    fontdict is a dictionary to override the default text properties.
+    If fontdict is None, the default is
 
       'fontsize'            : 9,
       'verticalalignment'   : 'bottom',
       'horizontalalignment' : 'left'
 
-    **kwargs can in turn be used to override the override, as in
+    **kwargs can in turn be used to override the fontdict, as in
 
       a.text(x,y,label, fontsize=12)
 
@@ -646,10 +937,10 @@ def text(x, y, label, *args, **kwargs):
     'verticalalignment'='bottom' and 'horizontalalignment'='left' but
     the keyword arg 'fontsize' will create a fontsize of 12
 
-    The purpose of all of these options is to make it easy for you to
-    create a default font theme for your plots by creating a single
-    dictionary, and then being able to selective change individual
-    attributes for the varous text creation commands, as in
+    The purpose these options is to make it easy for you to create a
+    default font theme for your plots by creating a single dictionary,
+    and then being able to selective change individual attributes for
+    the varous text creation commands, as in
 
         fonts = {
           'color'               : 'k',
@@ -662,7 +953,7 @@ def text(x, y, label, *args, **kwargs):
         ylabel('My ylabel', fonts, fontsize=10)
         text(12, 20, 'some text', fonts, fontsize=8)
 
-    The AxisText defaults are
+    The Text defaults are
 
         'color'               : 'k',
         'fontname'            : 'Sans',
@@ -672,9 +963,24 @@ def text(x, y, label, *args, **kwargs):
         'horizontalalignment' : 'left'
         'rotation'            : 'horizontal',
         'verticalalignment'   : 'bottom',
+        'transx'              : gca().xaxis.transData,
+        'transy'              : gca().yaxis.transData,            
+
+        transx and transy specify that text is in data coords,
+        alternatively, you can specify text in axis coords (0,0 lower
+        left and 1,1 upper right).  The example below places text in
+        the center of the axes
+
+        ax = subplot(111)
+        text(0.5, 0.5,'matplotlib', 
+             horizontalalignment='center',
+             verticalalignment='center',
+             transx = ax.xaxis.transAxis,
+             transy = ax.yaxis.transAxis,
+        )
 
     """
-    t =  gca().text(x, y, label, *args, **kwargs)
+    t =  gca().text(x, y, label, fontdict, **kwargs)
     draw_if_interactive()
     return t
 
@@ -758,146 +1064,3 @@ def ylabel(s, *args, **kwargs):
     draw_if_interactive()
     return l
 
-def grid(b):
-    """
-    Set the figure grid to be on or off (b is a boolean)
-    """
-    gca().grid(b)
-
-
-
-
-def legend(labels, loc='upper right'):
-    """
-    Place a legend on the current axes at location loc.  Labels are a
-    sequence of strings and loc can be a string or an integer
-    specifying the legend location
-
-    The location codes are
-    
-      'best' : 0,
-      'upper right' : 1,  (default)
-      'upper left'  : 2,
-      'lower left'  : 3,
-      'lower right' : 4,
-      'right'       : 5,
-
-    Return value is a sequence of text, line instances that make
-    up the legend
-
-    """
-    gca().legend(labels, loc)
-
-def semilogx(*args, **kwargs):
-    """
-    Make a semilog plot with log scaling on the x axis.  The args to
-    semilog x are the same as the args to plot.  See help plot for
-    more info    
-    """
-    a = gca()
-    a.set_xscale('log')
-    l = plot(*args, **kwargs)
-    return l
-
-
-def semilogy(*args, **kwargs):
-    """
-    Make a semilog plot with log scaling on the y axis.  The args to
-    semilog x are the same as the args to plot.  See help plot for
-    more info    
-    """
-    a = gca()
-    a.set_yscale('log')
-    l = plot(*args, **kwargs)
-    return l
-
-def loglog(*args, **kwargs):
-    """
-    Make a loglog plot with log scaling on the a and y axis.  The args
-    to semilog x are the same as the args to plot.  See help plot for
-    more info
-    """
-    a = gca()
-    a.set_xscale('log')
-    a.set_yscale('log')
-    l = plot(*args, **kwargs)
-    return l
-
-
-def pcolor(*args, **kwargs):
-    """
-    pcolor(C) - make a pseudocolor plot of matrix C
-
-    pcolor(X, Y, C) - a pseudo color plot of C on the matrices X and Y  
-
-    Shading:
-
-      The optional keyword arg shading ('flat' or 'faceted') will
-      determine whether the black grid is drawn around each pcolor
-      square.  Defaul 'faceteted'
-         e.g.,   
-         pcolor(C, shading='flat')  
-         pcolor(X, Y, C, shading='faceted')
-
-    returns a list of patch objects    
-    """
-    return gca().pcolor(*args, **kwargs)
-
-def xerrorbar(x, y, e, u=None, fmt='b-'):
-    """
-    Plot x versus y with error bars in e.  if u is not None, then u
-    gives the left error bars and e gives the right error bars.
-    Otherwise the error bars are symmetric about x and given in the
-    array e.
-
-    fmt is the plot format symbol for y
-
-    Return value is a length 2 tuple.  The first element is a list of
-    y symbol lines.  The second element is a list of error bar lines.
-
-    """
-    l0 = plot(x,y,fmt)
-
-    e = asarray(e)
-    if u is None: u = e
-    right = x+u
-    left = x-e
-    height = (max(y)-min(y))*0.001
-    a = gca()
-    try:
-        l1 = a.hlines(y, x, left)
-        l2 = a.hlines(y, x, right)
-        l3 = a.vlines(right, y-height, y+height)
-        l4 = a.vlines(left, y-height, y+height)
-    except ValueError, msg:
-        msg = raise_msg_to_str(msg)
-        error_msg(msg)
-        raise RuntimeError, msg
-
-    l1.extend(l2)
-    l3.extend(l4)
-    l1.extend(l3)
-    draw_if_interactive()
-    return (l0, l1)
-
-def xyerrorbar(x, y, e, f, u=None, v=None, fmt='b-'):
-    """
-    Plot x versus y with x-error bars in e and y-error bars in f.
-    If u is not None, then u gives the upper x-error bars and e gives the lower
-    x-error bars.
-    Otherwise the error bars are symmetric about y and given in the array e.
-    If v is not None, then v gives the upper y-error bars and f gives the lower
-    y-error bars.
-    Otherwise the error bars are symmetric about x and given in the array f.
-
-    fmt is the plot format symbol for the x,y point
-
-    Return value is a length 2 tuple.  The first element is a list of
-    y symbol lines.  The second element is a list of error bar lines.
-
-    """
-    errorbar(x, y, e, u, fmt)
-    xerrorbar(x, y, f, v, fmt)
-
-
-
